@@ -22,49 +22,32 @@
 #define _KMPLAYER_H_
 
 #include <libosso.h>
-#include "kmplayertypes.h"
-#include "kmplayerplaylist.h"
+
+#define MAEMO_NO_EXPORT __attribute__ ((visibility("hidden")))
 
 #define GCONF_KEY_PREFIX "/apps/maemo/kmplayer"
 #define GCONF_KEY_RECENT_FILE GCONF_KEY_PREFIX"/recent_file"
 #define GCONF_KEY_PLAYLIST_FILE GCONF_KEY_PREFIX"/playlist_file"
 #define GCONF_KEY_LOCATION_URL GCONF_KEY_PREFIX"/location"
 #define GCONF_KEY_PLAYER GCONF_KEY_PREFIX"/player"
+#define GCONF_KEY_MIME GCONF_KEY_PREFIX"/mimetypes/"
 
 #define HSPLIT 320
 #define VSPLIT 260
 
+#define CONTROL(app) ((KMPlayer::Control *)app->control)
+#define VIEWAREA(app) ((KMPlayer::ViewArea *)app->viewarea)
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef struct _HildonProgram HildonProgram;
 typedef struct _HildonWindow HildonWindow;
-typedef struct _GConfClient GConfClient;
-namespace KMPlayer {
-    struct PlayListData;
-    class Control;
-    class ViewArea;
-    class Node;
-    class Process;
-}
-
-class WaitNotify {
-    GtkWindow * window;
-public:
-    WaitNotify (GtkWindow * w, const KMPlayer::String & txt);
-    ~WaitNotify ();
-};
+/*typedef struct _GConfClient GConfClient;*/
 
 /* Application UI data struct */
-class Application {
-    KMPlayer::Config * m_config;
-public:
-    Application ();
-    ~Application ();
-    void initControl ();
-    void jump (KMPlayer::Node * n);
-    void fullScreen ();
-    void normalScreen ();
-    void setPlaying (bool play);
-    void setInfoContent (const char * utf8, bool plaintext);
-    KMPlayer::Config *config();
+typedef struct _Application {
     HildonProgram *program;
     HildonWindow *window;
     HildonWindow *window_fullscreen;
@@ -96,35 +79,18 @@ public:
     GtkPaned * hsplit_view;
     GtkPaned * vsplit_view;
     GtkWidget * isplit_view;
-    //GtkComboBoxEntry * location_bar;
     GtkCellRenderer * playlist_cell_renderer;
-    KMPlayer::PlayListData * playlist_data;
-    KMPlayer::Control * control;
-    KMPlayer::ViewArea * viewarea;
-    KMPlayer::NodePtr playlists;
-    KMPlayer::NodePtr recents;
-    KMPlayer::String url;
-    KMPlayer::String copyPasteUrl;
-    KMPlayer::NodePtr copyPasteNode;
-    KMPlayer::Process * media_server;
-    KMPlayer::Process * mplayer;
-#ifdef __ARMEL__
-    KMPlayer::Process * npp;
-#endif
+    void *control;
+    void *viewarea;
     short playlists_tree_id;
     short recents_tree_id;
-    bool in_tree_update; // no events when updating
+    bool in_tree_update; /* no events when updating*/
     bool fullscreen;
     bool playing;
-};
+} Application;
 
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 GtkWidget * createTreeView (Application * app);
-short addTree (Application * a, KMPlayer::NodePtr doc, GdkPixbuf * ico);
 void togglePlaylistShow (GtkCheckMenuItem *checkmenuitem, Application * app);
 void togglePlaylistShowAll (GtkCheckMenuItem *checkmenuitem, Application * app);
 void togglePlaylistEdit (GtkCheckMenuItem *checkmenuitem, Application * app);
@@ -146,26 +112,34 @@ void cb_playlistCopy (GtkButton * button, Application * app);
 void cb_playlistPaste (GtkButton * button, Application * app);
 void cb_playlistPasteInNewGroup (GtkButton * button, Application * app);
 void cb_playlistCut (GtkButton * button, Application * app);
-void cb_playlist_prev (GtkButton * button, Application * app);
-void cb_playlist_play (GtkButton * button, Application * app);
-void cb_playlist_next (GtkButton * button, Application * app);
-void cb_playlist_stop (GtkButton * button, Application * app);
+void cb_playlist_prev (GtkButton * button, void *ctr);
+void cb_playlist_play (GtkButton * button, void *ctr);
+void cb_playlist_next (GtkButton * button, void *ctr);
+void cb_playlist_stop (GtkButton * button, void *ctr);
 void progressValueChanged(GtkWidget *w, Application * app);
 gboolean cb_scheduledPaint (void * p);
 gboolean cbViewAreaFullscreenMapped (GtkWidget *w, GdkEvent *, Application *a);
 void cbViewAreaFullscreenUnmapped (GtkWidget *w, Application *a);
+
+void save_locationbar (Application * app);
+gboolean locationbarGainFocus (GtkWidget *, GdkEventFocus *, Application *app);
+
+Application *app_new () MAEMO_NO_EXPORT;
+void app_destroy (Application *app) MAEMO_NO_EXPORT;
+void app_init_control (Application *app) MAEMO_NO_EXPORT;
+void app_insert_location (Application *app, const char *loc) MAEMO_NO_EXPORT;
+void app_open_location (const char *filename, Application *app) MAEMO_NO_EXPORT;
+void app_normal_screen (Application *app) MAEMO_NO_EXPORT;
+void app_set_info_content (Application *app, const char * utf8, bool plaintext) MAEMO_NO_EXPORT;
+void app_set_playing (Application *app, bool) MAEMO_NO_EXPORT;
+void app_set_title (const char *title, Application *app) MAEMO_NO_EXPORT;
+
+void openFile (Application *app, const gchar *path) MAEMO_NO_EXPORT;
+const char *saveFile (void *ctr, const gchar *path) MAEMO_NO_EXPORT;
+void newFile (Application *app) MAEMO_NO_EXPORT;
+
 #ifdef __cplusplus
 }
 #endif
 
-void updateTree (Application * app, short id);
-void rowSelected (Application * app);
-void openFile (Application * app, const gchar * path);
-void insertLocation (Application * app, const KMPlayer::String & loc);
-void openLocation (const char * filename, Application * app);
-void setTitle (const char *, Application * app);
-void openDocument(Application *app,KMPlayer::NodePtr doc,KMPlayer::NodePtr cur);
-bool saveFile (Application * app, const gchar * path);
-void newFile (Application * app);
-
 #endif //_KMPLAYER_H_
