@@ -22,6 +22,7 @@
 
 #include "kmplayer_lists.h"
 #include "kmplayer.h"
+#include "kmplayercontrol.h"
 
 using namespace KMPlayer;
 
@@ -72,7 +73,7 @@ KDE_NO_EXPORT void Recents::activate () {
 KDE_NO_EXPORT void Recents::defer () {
     if (!resolved) {
         resolved = true;
-        readFromFile (app->config ()->readEntry
+        readFromFile (CONTROL(app)->m_config->readEntry
                 (GCONF_KEY_RECENT_FILE, "/home/user/recent.xml"));
     }
 }
@@ -103,7 +104,7 @@ KDE_NO_EXPORT void Recent::closed () {
 }
 
 KDE_NO_EXPORT void Recent::activate () {
-    openLocation (src, app);
+    app_open_location ((const char *) src, app);
 }
 
 KDE_NO_CDTOR_EXPORT
@@ -133,7 +134,7 @@ KDE_NO_EXPORT void Playlist::defer () {
         KMPlayer::Document::defer ();
     else if (!resolved) {
         resolved = true;
-        readFromFile (app->config ()->readEntry
+        readFromFile (CONTROL(app)->m_config->readEntry
                 (GCONF_KEY_PLAYLIST_FILE, "/home/user/playlist.xml"));
     }
 }
@@ -156,9 +157,7 @@ KDE_NO_EXPORT KMPlayer::NodePtr Playlist::childFromTag (const String & tag) {
     // kdDebug () << nodeName () << " childFromTag " << tag << endl;
     const char * name = (const char *) tag;
     if (!strcmp (name, "item"))
-        return playmode
-            ? KMPlayer::NodePtr (new KMPlayer::GenericMrl (m_doc, String(), String(), "item"))
-            : KMPlayer::NodePtr (new PlaylistItem (m_doc, app));
+        return new PlaylistItem (m_doc, app, playmode);
     else if (!strcmp (name, "group"))
         return new PlaylistGroup (m_doc, app, playmode);
     else if (!strcmp (name, "object"))
@@ -174,36 +173,41 @@ KDE_NO_EXPORT void Playlist::childDone (KMPlayer::NodePtr c) {
 }
 
 KDE_NO_CDTOR_EXPORT
-PlaylistItemBase::PlaylistItemBase (KMPlayer::NodePtr &d, short i, Application *a)
-  : KMPlayer::Mrl (d, i), app (a) {
+PlaylistItemBase::PlaylistItemBase (KMPlayer::NodePtr &d, short i, Application *a, bool pm)
+  : KMPlayer::Mrl (d, i), app (a), playmode (pm) {
 }
 
 KDE_NO_EXPORT void PlaylistItemBase::activate () {
-    NodePtr pl = new Playlist (app, 0L, true);
-    String data;
-    String pn;
-    if (parentNode ()->id == KMPlayer::id_node_group_node &&
-            (previousSibling() || nextSibling ())) {
-        data = parentNode ()->innerXML ();
-        pn = parentNode ()->mrl ()->pretty_name;
+    if (playmode) {
+        Mrl::activate ();
     } else {
-        data = outerXML ();
-        pn = pretty_name.isEmpty () ? src : pretty_name;
-    }
-    pl->mrl ()->pretty_name = pn;
-    debugLog () << "cloning to " << data << endl;
-    TextStream inxml (data);
-    KMPlayer::readXML (pl, inxml, String(), false);
-    pl->normalize ();
-    NodePtr cur = pl->firstChild ();
-    if (parentNode ()->id == KMPlayer::id_node_group_node && cur) {
-        KMPlayer::NodePtr sister = parentNode ()->firstChild ();
-        while (sister && cur && sister.ptr () != this) {
-            sister = sister->nextSibling ();
-            cur = cur->nextSibling ();
+        NodePtr pl = new Playlist (app, 0L, true);
+        String data;
+        String pn;
+        if (parentNode ()->id == KMPlayer::id_node_group_node &&
+                (previousSibling() || nextSibling ())) {
+            data = parentNode ()->innerXML ();
+            pn = parentNode ()->mrl ()->pretty_name;
+        } else {
+            data = outerXML ();
+            pn = pretty_name.isEmpty () ? src : pretty_name;
         }
+        pl->mrl ()->pretty_name = pn;
+        //debugLog () << "cloning to " << data << endl;
+        TextStream inxml (data);
+        KMPlayer::readXML (pl, inxml, String(), false);
+        pl->normalize ();
+        NodePtr cur = pl->firstChild ();
+        pl->mrl ()->resolved = !!cur;
+        if (parentNode ()->id == KMPlayer::id_node_group_node && cur) {
+            KMPlayer::NodePtr sister = parentNode ()->firstChild ();
+            while (sister && cur && sister.ptr () != this) {
+                sister = sister->nextSibling ();
+            cur = cur->nextSibling ();
+            }
+        }
+        CONTROL(app)->openDocument (pl, cur);
     }
-    openDocument (app, pl, cur);
 }
 
 KDE_NO_EXPORT void PlaylistItemBase::closed () {
@@ -212,8 +216,8 @@ KDE_NO_EXPORT void PlaylistItemBase::closed () {
 }
 
 KDE_NO_CDTOR_EXPORT
-PlaylistItem::PlaylistItem (KMPlayer::NodePtr & doc, Application * a, const String &url)
-  : PlaylistItemBase (doc, KMPlayer::id_node_playlist_item, a) {
+PlaylistItem::PlaylistItem (KMPlayer::NodePtr & doc, Application * a, bool pm, const String &url)
+  : PlaylistItemBase (doc, KMPlayer::id_node_playlist_item, a, pm) {
     src = url;
     setAttribute (KMPlayer::StringPool::attr_url, url);
 }
@@ -224,6 +228,13 @@ KDE_NO_EXPORT void PlaylistItem::closed () {
     PlaylistItemBase::closed ();
 }
 
+KDE_NO_EXPORT void PlaylistItem::begin () {
+    if (playmode && firstChild ())
+        firstChild ()->activate ();
+    else
+        Mrl::begin ();
+}
+
 KDE_NO_EXPORT void PlaylistItem::setNodeName (const String & s) {
     src = s;
     setAttribute (KMPlayer::StringPool::attr_url, s);
@@ -245,9 +256,7 @@ PlaylistGroup::PlaylistGroup (KMPlayer::NodePtr & doc, Application * a, bool lm)
 KDE_NO_EXPORT KMPlayer::NodePtr PlaylistGroup::childFromTag (const String & tag) {
     const char * name = (const char *) tag;
     if (!strcmp (name, "item"))
-        return playmode
-            ? KMPlayer::NodePtr (new KMPlayer::GenericMrl (m_doc, String(), String(), "item"))
-            : KMPlayer::NodePtr (new PlaylistItem (m_doc, app));
+        return new PlaylistItem (m_doc, app, playmode);
     else if (!strcmp (name, "group"))
         return new PlaylistGroup (m_doc, app, playmode);
     else if (!strcmp (name, "object"))
@@ -267,7 +276,7 @@ KDE_NO_EXPORT void PlaylistGroup::setNodeName (const String & t) {
 
 KDE_NO_CDTOR_EXPORT
 HtmlObject::HtmlObject (KMPlayer::NodePtr &doc, Application *a, bool pm)
-  : PlaylistItemBase (doc, KMPlayer::id_node_html_object, a), playmode (pm) {}
+  : PlaylistItemBase (doc, KMPlayer::id_node_html_object, a, pm) {}
 
 KDE_NO_EXPORT void HtmlObject::activate () {
     if (playmode)
@@ -304,5 +313,6 @@ KDE_NO_EXPORT KMPlayer::NodePtr HtmlObject::childFromTag (const String & tag) {
         return new DarkNode (m_doc, name, KMPlayer::id_node_param);
     else if (!strcasecmp (name, "embed"))
         return new DarkNode (m_doc, name, id_node_html_embed);
+    return NULL;
 }
 
