@@ -25,25 +25,19 @@
 
 #include "kmplayerplaylist.h"
 
-#include <cairo.h>
-
 namespace KIO {
     class Job;
 }
 
+struct TransTypeInfo;
+
 namespace KMPlayer {
 
 struct KMPLAYER_NO_EXPORT ImageData {
     ImageData( const String & img);
     ~ImageData();
-    bool isEmpty ();
-    Single width ();
-    Single height ();
-    cairo_pattern_t * cairoImage (Single w, Single h, cairo_surface_t *cs);
-    cairo_pattern_t * cairoImage (cairo_surface_t * cs);
-    cairo_pattern_t * cairo_image;
-    Image * image;
-    Single w, h;
+    Image *image;
+private:
     String url;
 };
 
@@ -52,6 +46,7 @@ typedef WeakPtr <ImageData> ImageDataPtrW;
 
 struct KMPLAYER_NO_EXPORT CachedImage {
     void setUrl (const String & url);
+    bool isEmpty ();
     ImageDataPtr data;
 };
 
@@ -76,11 +71,17 @@ public:
     SizeType (const String & s);
     void reset ();
     SizeType & operator = (const String & s);
-    Single size (Single relative_to = 100);
+    SizeType & operator += (const SizeType & s);
+    SizeType & operator -= (const SizeType & s);
+    SizeType & operator /= (const int i)
+        { perc_size /= i; abs_size /= i; return *this; }
+    SizeType & operator *= (const float f)
+        { perc_size *= f; abs_size *= f; return *this; }
+    Single size (Single relative_to = 100) const;
     bool isSet () const { return isset; }
 private:
-    Single m_size;
-    bool percentage;
+    Single perc_size;
+    Single abs_size;
     bool isset;
 };
 
@@ -99,7 +100,8 @@ public:
             Single & xoff, Single & yoff, Single & w1, Single & h1);
     SizeType left, top, width, height, right, bottom;
     String reg_point, reg_align;
-    bool setSizeParam (const TrieString & name, const String & value);
+    bool setSizeParam (const TrieString &name, const String &value, bool &dim);
+    void move (const SizeType &x, const SizeType &y);
 };
 
 /**
@@ -165,17 +167,13 @@ public:
     ~MediaTypeRuntime ();
     virtual void reset ();
     virtual void stopped ();
-    virtual bool parseParam (const TrieString & name, const String & value);
     virtual void postpone (bool b);
     virtual void clipStart ();
     virtual void clipStop ();
-    virtual SRect intrinsicBounds ();
-    CalculatedSizer sizes;
     PostponePtr postpone_lock;
-    Fit fit;
     MediaTypeRuntime (NodePtr e);
 protected:
-    ConnectionPtr document_postponed;      // pauze audio/video accordantly
+    ConnectionPtr document_postponed;      // pause audio/video accordantly
 };
 
 /**
@@ -203,7 +201,6 @@ public:
     virtual void postpone (bool b);
     virtual void clipStart ();
     virtual void clipStop ();
-    virtual SRect intrinsicBounds ();
     CachedImage cached_img;
     bool have_frame;
 protected:
@@ -282,7 +279,7 @@ private:
     enum { acc_none, acc_sum } accumulate;
     enum { add_replace, add_sum } additive;
     int change_by;
-    enum { calc_discrete, calc_linear, calc_paced } calcMode;
+    enum { calc_discrete, calc_linear, calc_paced, calc_spline } calcMode;
     String change_from;
     StringList change_values;
     int steps;
@@ -290,6 +287,44 @@ private:
     String change_from_unit;
 };
 
+/**
+ * Stores runtime data of animate element
+ */
+class KMPLAYER_NO_EXPORT AnimateMotionData : public AnimateGroupData {
+public:
+    AnimateMotionData (NodePtr e);
+    ~AnimateMotionData ();
+    virtual bool parseParam (const TrieString & name, const String & value);
+    virtual void reset ();
+    virtual void started ();
+    virtual void stopped ();
+    bool timerTick();
+private:
+    bool checkTarget (Node *n);
+    bool setInterval ();
+    void applyStep ();
+    bool getCoordinates (const String &coord, SizeType &x, SizeType &y);
+    TimerInfoPtrW anim_timer;
+    enum { acc_none, acc_sum } accumulate;
+    enum { add_replace, add_sum } additive;
+    enum { calc_discrete, calc_linear, calc_paced, calc_spline } calcMode;
+    String change_from;
+    String change_by;
+    StringList values;
+    float *keytimes;
+    int keytime_count;
+    StringList splines;
+    float control_point[4];
+    unsigned int steps;
+    unsigned int cur_step;
+    unsigned int keytime_steps;
+    unsigned int interval;
+    SizeType begin_x, begin_y;
+    SizeType cur_x, cur_y;
+    SizeType delta_x, delta_y;
+    SizeType end_x, end_y;
+};
+
 class KMPLAYER_NO_EXPORT MouseListeners {
 public:
     MouseListeners();
@@ -358,6 +393,7 @@ public:
     void closed ();
     void childDone (NodePtr child);
     bool expose () const;
+    bool handleEvent (EventPtr event);
     void accept (Visitor *);
     void jump (const String & id);
     static Smil * findSmilNode (Node * node);
@@ -387,16 +423,16 @@ public:
 /**
  * Base class for SMIL::Region, SMIL::RootLayout and SMIL::Layout
  */
-class RegionBase : public Element {
+class KMPLAYER_NO_EXPORT RegionBase : public RemoteObject, public Element {
 public:
+    enum ShowBackground { ShowAlways, ShowWhenActive };
+
     ~RegionBase ();
     bool expose () const { return false; }
     void activate ();
     void childDone (NodePtr child);
     void deactivate ();
-    virtual bool handleEvent (EventPtr event);
     virtual void parseParam (const TrieString & name, const String & value);
-    virtual NodeRefListPtr listeners (unsigned int event_id);
     /**
      * repaints region, calls scheduleRepaint(x,y,w,h) on view
      */
@@ -407,17 +443,23 @@ public:
      * given this element's w and h value
      * and child's left/top/right/width/height/bottom attributes
      */
-    virtual void updateDimensions (SurfacePtr parent_surface);
+    virtual void updateDimensions ();
+    void boundsUpdate (); // recalculates and repaint old and new bounds
 
-    SurfacePtrW surface;
+    virtual Surface *surface ();
+    SurfacePtrW region_surface;
+    CachedImage cached_img;
     CalculatedSizer sizes;
 
     Single x, y, w, h;     // unscaled values
     int z_order;
     unsigned int background_color;
+    String background_image;
+    ShowBackground show_background;
 protected:
     RegionBase (NodePtr & d, short id);
-    NodeRefListPtr m_SizeListeners;        // region resized
+    PostponePtr postpone_lock;               // pause while loading bg image
+    virtual void remoteReady (ByteArray &);  // image downloaded
 };
 
 /**
@@ -430,12 +472,12 @@ public:
     KDE_NO_EXPORT const char * nodeName () const { return "layout"; }
     void activate ();
     void closed ();
-    virtual bool handleEvent (EventPtr event);
     virtual void accept (Visitor *);
     /**
      * recursively calculates dimensions of this and child regions
      */
-    virtual void updateDimensions (SurfacePtr parent_surface);
+    virtual void updateDimensions ();
+    virtual Surface *surface ();
 
     NodePtrW rootLayout;
 };
@@ -449,16 +491,15 @@ public:
     KDE_NO_EXPORT const char * nodeName () const { return "region"; }
     NodePtr childFromTag (const String & tag);
     void calculateBounds (Single w, Single h);
-    virtual bool handleEvent (EventPtr event);
     virtual NodeRefListPtr listeners (unsigned int event_id);
     virtual void accept (Visitor *);
     /**
      * boolean for check if pointerEntered/pointerLeft should be called by View
      */
     bool has_mouse;
+    NodeRefListPtr m_AttachedMediaTypes;   // active attached mediatypes
 private:
     MouseListeners mouse_listeners;
-    NodeRefListPtr m_AttachedMediaTypes;   // active attached mediatypes
 };
 
 /**
@@ -489,17 +530,48 @@ public:
  */
 class KMPLAYER_NO_EXPORT Transition : public Element {
 public:
+    enum TransType {
+        TransTypeNone = 0,
+        BarWipe, IrisWipe, ClockWipe, SnakeWipe, // required, TODO
+        BoxWipe, FourBoxWipe, BarnDoorWipe, DiagonalWipe, BowTieWipe,
+        MiscDiagonalWipe, VeeWipe, BarnVeeWipe, ZigZagWipe, BarnZigZagWipe,
+        TriangleWipe, ArrowHeadWipe, PentagonWipe, HexagonWipe, EllipseWipe,
+        EyeWipe, RoundRectWipe, StarWipe, MiscShapeWipe,
+        PinWheelWipe, SingleSweepWipe, FanWipe, DoubleFanWipe,
+        DoubleSweepWipe, SaloonDoorWipe, WindShieldWipe,
+        SpiralWipe, ParallelSnakesWipe, BoxSnakesWipe, WaterFallWipe,
+        PushWipe, SideWipe, Fade,
+        TransLast
+    };
+    enum TransSubType {
+        SubTransTypeNone = 0,
+        SubLeftToRight, SubTopToBottom, SubTopLeft, SubTopRight,
+        SubBottomRight, SubBottomLeft,
+        SubTopCenter, SubRightCenter, SubBottomCenter, SubLeftCenter,
+        SubCornersIn, SubCornersOut,
+        SubCircle, SubVertical, SubHorizontal,
+        SubFromLeft, SubFromTop, SubFromRight, SubFromBottom,
+        SubCrossfade, SubFadeToColor, SubFadeFromColor,
+        SubRectangle, SubDiamond,
+        SubClockwiseTwelve, SubClockwiseThree, SubClockwiseSix,
+        SubClockwiseNine,
+         // and lots more .. TODO
+        SubTransLast
+    };
     Transition (NodePtr & d);
     KDE_NO_CDTOR_EXPORT ~Transition () {}
     void activate ();
+    KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
     KDE_NO_EXPORT const char * nodeName () const { return "transition"; }
     void parseParam (const TrieString & name, const String & value);
     KDE_NO_EXPORT bool expose () const { return false; }
     bool supported ();
-    String type;
-    String subtype;
+    TransType type;
+    TransSubType sub_type;
+    TransTypeInfo *type_info;
     enum { dir_forward, dir_reverse } direction;
-    unsigned int dur; // deci seconds
+    int dur; // deci seconds
+    float start_progress, end_progress;
     unsigned int fade_color;
 };
 
@@ -519,7 +591,7 @@ public:
     void finish ();
     void deactivate ();
     void reset ();
-    bool expose () const;
+    bool expose () const { return false; }
     void childBegan (NodePtr child);
     void childDone (NodePtr child);
     virtual bool handleEvent (EventPtr event);
@@ -529,8 +601,8 @@ public:
     virtual void parseParam (const TrieString &, const String &);
     Runtime * runtime ();
     static Runtime::DurationItem * getDuration (NodePtr n);
-    static bool isTimedMrl (const NodePtr & n);
-    static bool keepContent (NodePtr n);
+    static bool isTimedMrl (const Node *n);
+    static bool keepContent (Node *n);
     static Fill getDefaultFill (NodePtr n);
     unsigned int begin_time;
     unsigned int finish_time;
@@ -553,7 +625,7 @@ KDE_NO_EXPORT inline Runtime * TimedMrl::runtime () {
     return m_runtime;
 }
 
-KDE_NO_EXPORT inline bool TimedMrl::isTimedMrl (const NodePtr & n) {
+KDE_NO_EXPORT inline bool TimedMrl::isTimedMrl (const Node *n) {
     return n &&
         n->id >= id_node_first_timed_mrl &&
         n->id <= id_node_last_timed_mrl;
@@ -565,6 +637,7 @@ KDE_NO_EXPORT inline bool TimedMrl::isTimedMrl (const NodePtr & n) {
 class GroupBase : public TimedMrl {
 public:
     KDE_NO_CDTOR_EXPORT ~GroupBase () {}
+    NodePtr childFromTag (const String & tag);
     PlayType playType () { return play_type_none; }
     void finish ();
     void deactivate ();
@@ -580,7 +653,6 @@ protected:
 class Par : public GroupBase {
 public:
     KDE_NO_CDTOR_EXPORT Par (NodePtr & d) : GroupBase (d, id_node_par) {}
-    NodePtr childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "par"; }
     void begin ();
     void reset ();
@@ -593,7 +665,6 @@ public:
 class Seq : public GroupBase {
 public:
     KDE_NO_CDTOR_EXPORT Seq (NodePtr & d) : GroupBase(d, id_node_seq) {}
-    NodePtr childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "seq"; }
     void begin ();
     void childDone (NodePtr child);
@@ -617,7 +688,6 @@ public:
 class Excl : public GroupBase {
 public:
     KDE_NO_CDTOR_EXPORT Excl (NodePtr & d) : GroupBase (d, id_node_excl) {}
-    NodePtr childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "excl"; }
     void begin ();
     void deactivate ();
@@ -634,15 +704,12 @@ private:
 class Switch : public GroupBase {
 public:
     KDE_NO_CDTOR_EXPORT Switch (NodePtr &d) : GroupBase (d, id_node_switch) {}
-    NodePtr childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "switch"; }
-    PlayType playType ();
     // Condition
-    void activate ();
+    void begin ();
     void deactivate ();
     void reset ();
     void childDone (NodePtr child);
-    Mrl * linkNode ();
     NodePtrW chosenOne;
 };
 
@@ -694,37 +761,47 @@ public:
     MediaType (NodePtr & d, const String & t, short id);
     NodePtr childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return (const char*)m_type; }
+    void closed ();
     void activate ();
     void deactivate ();
     void begin ();
     void finish ();
-    bool expose () const;
     void childDone (NodePtr child);
     virtual SurfacePtr getSurface (NodePtr node);
+    /* (new) sub-region or NULL if not displayed */
+    Surface *surface ();
+    void resetSurface ();
+    SRect calculateBounds ();
+    void boundsUpdate (); // recalculates and repaint old and new bounds
     virtual void parseParam (const TrieString & name, const String & value);
     virtual bool handleEvent (EventPtr event);
     NodeRefListPtr listeners (unsigned int event_id);
-    void positionVideoWidget (); // for 'video' and 'ref' nodes
+    bool needsVideoWidget (); // for 'video' and 'ref' nodes
+    SurfacePtrW sub_surface;
     NodePtrW external_tree; // if src points to playlist, the resolved top node
     NodePtrW trans_in;
     NodePtrW trans_out;
+    NodePtrW active_trans;
     NodePtrW region_node;
-    SRect cached_rect; // rel. rect wrt. region, eg. fit and regpoints applied
     String m_type;
+    CalculatedSizer sizes;
+    Fit fit;
+    int opacity;
     unsigned int bitrate;
     unsigned int trans_step;
     unsigned int trans_steps;
     enum { sens_opaque, sens_transparent, sens_percentage } sensitivity;
+    bool trans_out_active;
 protected:
     MouseListeners mouse_listeners;
     NodeRefListPtr m_MediaAttached;        // mouse entered
-    ConnectionPtr region_sized;            // attached region is sized
     ConnectionPtr region_paint;            // attached region needs painting
     ConnectionPtr region_mouse_enter;      // attached region has mouse entered
     ConnectionPtr region_mouse_leave;      // attached region has mouse left
     ConnectionPtr region_mouse_click;      // attached region is clicked
     ConnectionPtr region_attach;           // attached to region
     TimerInfoPtrW trans_timer;
+    TimerInfoPtrW trans_out_timer;
 };
 
 class AVMediaType : public MediaType {
@@ -735,8 +812,8 @@ public:
     virtual void defer ();
     virtual void undefer ();
     virtual void endOfFile ();
-    virtual bool handleEvent (EventPtr event);
     virtual void accept (Visitor *);
+    virtual bool expose () const;
 };
 
 class ImageMediaType : public MediaType {
@@ -761,7 +838,6 @@ class RefMediaType : public MediaType {
 public:
     RefMediaType (NodePtr & d);
     Runtime * getNewRuntime ();
-    virtual bool handleEvent (EventPtr event);
     virtual void accept (Visitor *);
 };
 
@@ -789,6 +865,18 @@ public:
     bool handleEvent (EventPtr event);
 };
 
+class KMPLAYER_NO_EXPORT AnimateMotion : public TimedMrl {
+public:
+    KDE_NO_CDTOR_EXPORT AnimateMotion (NodePtr & d)
+        : TimedMrl (d, id_node_animate) {}
+    KDE_NO_EXPORT const char * nodeName () const { return "animateMotion"; }
+    virtual Runtime *getNewRuntime ();
+    PlayType playType () { return play_type_none; }
+    bool handleEvent (EventPtr event);
+};
+
+// TODO animateColor transitionFilter
+
 class Param : public Element {
 public:
     KDE_NO_CDTOR_EXPORT Param (NodePtr & d) : Element (d, id_node_param) {}
