@@ -24,6 +24,7 @@
 #ifdef HAVE_EXPAT
 #include <expat.h>
 #endif
+#include <cairo.h>
 #include "kmplayerplaylist.h"
 #include "kmplayer_asx.h"
 #include "kmplayer_atom.h"
@@ -112,14 +113,14 @@ KDE_NO_EXPORT void Connection::disconnect () {
 KMPLAYER_NO_CDTOR_EXPORT
 TimerInfo::TimerInfo (NodePtr n, unsigned id, struct timeval & tv, int ms)
  : node (n), event_id (id), timeout (tv), milli_sec (ms) {}
-    
+
 //-----------------------------------------------------------------------------
 
 Matrix::Matrix () : a (1.0), b (0.0), c (0.0), d (1.0), tx (0), ty (0) {}
 
 Matrix::Matrix (const Matrix & m)
  : a (m.a), b (m.b), c (m.c), d (m.d), tx (m.tx), ty (m.ty) {}
-    
+
 Matrix::Matrix (Single xoff, Single yoff, float xscale, float yscale)
  : a (xscale), b (0.0), c (0.0), d (yscale), tx (xoff), ty (yoff) {}
 
@@ -134,6 +135,17 @@ void Matrix::getXYWH (Single & x, Single & y, Single & w, Single & h) const {
     h *= d;
 }
 
+void Matrix::invXYWH (Single & x, Single & y, Single & w, Single & h) const {
+    if (a > 0.00001 && d > 0.00001) {
+        w /= a;
+        h /= d;
+        x = Single ((x - tx) / a);
+        y = Single ((y - ty) / d);
+    } else {
+        warningLog () << "Not invering " << a << ", " << d << " scale" << endl;
+    }
+}
+
 void Matrix::transform (const Matrix & matrix) {
     // TODO: rotate
     a *= matrix.a;
@@ -480,7 +492,7 @@ void RefNode::setRefNode (const NodePtr ref) {
 
 //-----------------------------------------------------------------------------
 
-namespace KMPlayer {
+namespace {
     struct KMPLAYER_NO_EXPORT ParamValue {
         String val;
         StringList  * modifications;
@@ -490,6 +502,9 @@ namespace KMPlayer {
         void setValue (const String & v) { val = v; }
     };
     typedef std::map <TrieString, ParamValue *> ParamMap;
+}
+
+namespace KMPlayer {
     class KMPLAYER_NO_EXPORT ElementPrivate {
     public:
         ~ElementPrivate ();
@@ -634,7 +649,7 @@ static bool hasMrlChildren (const NodePtr & e) {
 
 Mrl::Mrl (NodePtr & d, short id)
     : Element (d, id), cached_ismrl_version (~0),
-      width (0), height (0), aspect (0), repeat (0),
+      aspect (0), repeat (0),
       view_mode (SingleMode),
       resolved (false), bookmarkable (true),
       audio_only (false) {}
@@ -701,8 +716,12 @@ void Mrl::activate () {
         Element::activate ();
         return;
     }
-    //kdDebug () << nodeName () << " Mrl::activate" << endl;
     setState (state_activated);
+    begin ();
+}
+
+void Mrl::begin () {
+    //kdDebug () << nodeName () << " Mrl::activate" << endl;
     if (document ()->notify_listener) {
         if (linkNode () != this) {
             linkNode ()->activate ();
@@ -744,11 +763,32 @@ void Mrl::parseParam (const TrieString & para, const String & val) {
 }
 
 KDE_NO_CDTOR_EXPORT Surface::Surface (NodePtr n, const SRect & r)
-  : node (n), bounds (r), xscale (1.0), yscale (1.0), background_color (0) {}
+  : node (n),
+    bounds (r),
+    xscale (1.0),
+    yscale (1.0),
+    background_color (0),
+    dirty (false),
+    surface (0L)
+{}
 
-Surface::Surface (const SRect & r) : bounds (r) {}
+Surface::~Surface() {
+    if (surface)
+        cairo_surface_destroy (surface);
+}
 
-Surface::~Surface() {}
+void Surface::remove () {
+    Surface *sp = parentNode ().ptr ();
+    if (sp) {
+        sp->markDirty ();
+        sp->removeChild (this);
+    }
+}
+
+void Surface::markDirty () {
+    for (Surface *s = this; s; s = s->parentNode ().ptr ())
+        s->dirty = true;
+}
 
 //-----------------------------------------------------------------------------
 
@@ -764,7 +804,7 @@ Postpone::~Postpone () {
 
 //-----------------------------------------------------------------------------
 
-namespace KMPlayer {
+namespace {
     static NodePtr dummy_element;
 }
 
@@ -828,7 +868,7 @@ void Document::activate () {
 }
 
 void Document::defer () {
-    if (!firstChild () || firstChild ()->state > state_init)
+    if (resolved)
         postpone_lock = postpone ();
     Mrl::defer ();
 }
@@ -930,8 +970,13 @@ bool Document::timer () {
                 TimerInfoPtr tinfo2 (tinfo); // prevent destruction
                 timers.remove (tinfo);
                 timeOfDay (now);
+                int drift = diffTime (now, tinfo2->timeout);
+                if (drift > tinfo2->milli_sec) {
+                    drift = tinfo2->milli_sec;
+                    //warningLog() << "time drift" << endl;
+                }
                 tinfo2->timeout = now;
-                addTime (tinfo2->timeout, tinfo2->milli_sec);
+                addTime (tinfo2->timeout, tinfo2->milli_sec - drift);
                 TimerInfoPtr ti = timers.first ();
                 int pos = 0;
                 for (; ti && diffTime (tinfo2->timeout, ti->timeout) >= 0; ti = ti->nextSibling ()) {
@@ -1094,7 +1139,7 @@ bool GenericMrl::expose () const {
 
 namespace KMPlayer {
 
-class DocumentBuilder {
+class KMPLAYER_NO_EXPORT DocumentBuilder {
     int m_ignore_depth;
     bool m_set_opener;
     bool m_root_is_first;
@@ -1294,7 +1339,7 @@ void readXML (NodePtr root, TextStream & in, const String & firstline, bool set_
 //-----------------------------------------------------------------------------
 #else // HAVE_EXPAT
 
-namespace KMPlayer {
+namespace {
 
 class SimpleSAXParser {
 public:
@@ -1352,8 +1397,10 @@ private:
     void push_attribute ();
 };
 
+} // namespace
+
 KMPLAYER_EXPORT
-void readXML (NodePtr root, TextStream & in, const String & firstline, bool set_opener) {
+void KMPlayer::readXML (NodePtr root, TextStream & in, const String & firstline, bool set_opener) {
     DocumentBuilder builder (root, set_opener);
     SimpleSAXParser parser (builder);
     if (!firstline.isEmpty ()) {
@@ -1369,8 +1416,6 @@ void readXML (NodePtr root, TextStream & in, const String & firstline, bool set_
     //kdDebug () << root->outerXML ();
 }
 
-} // namespace
-
 void SimpleSAXParser::push () {
     if (next_token->string.length ()) {
         prev_token = token;
@@ -1468,7 +1513,7 @@ bool SimpleSAXParser::nextToken () {
                     }
                     //kdDebug () << "entity found "<<prev_token->string << endl;
                 } else if (token->token == tok_hash &&
-                        nextToken () && token->token == tok_text && 
+                        nextToken () && token->token == tok_text &&
                         nextToken () && token->token == tok_semi_colon) {
                     //kdDebug () << "char entity found " << prev_token->string << prev_token->string.toInt (0L, 16) << endl;
                     token->token = tok_text;
