@@ -29,6 +29,7 @@
 #include "kmplayertypes.h"
 #include "kmplayershared.h"
 
+typedef struct _cairo_surface cairo_surface_t;
 
 namespace KMPlayer {
 
@@ -224,16 +225,6 @@ protected:
 
 ITEM_AS_POINTER(KMPlayer::Event)
 
-/**
- * Event signaling that attached region is resized
- */
-class SizeEvent : public Event {
-public:
-    SizeEvent (Single x, Single y, Single w, Single h, Fit f);
-    Single x, y, w, h;
-    Fit fit;
-};
-
 extern const unsigned int event_pointer_clicked;
 extern const unsigned int event_pointer_moved;
 extern const unsigned int event_inbounds;
@@ -545,6 +536,7 @@ public:
      * Reimplement to callback with requestPlayURL if isPlayable()
      */ 
     virtual void activate ();
+    virtual void begin ();
     /**
      * By default support one event handler (eg. SMIL or RP child document)
      */
@@ -559,8 +551,8 @@ public:
     String src;
     String pretty_name;
     String mimetype;
-    int width;
-    int height;
+    Single width;
+    Single height;
     float aspect;
     int repeat;
     enum { SingleMode = 0, WindowMode } view_mode;
@@ -637,14 +629,18 @@ public:
     virtual SurfacePtr createSurface (NodePtr owner, const SRect & rect) = 0;
     virtual void toScreen (Single & x, Single & y, Single & w, Single & h) = 0;
     virtual void resize (const SRect & rect) = 0;
-    virtual void repaint (Single x, Single y, Single w, Single h) = 0;
-    virtual void video (Single x, Single y, Single w, Single h) = 0;
+    virtual void repaint () = 0;
+    virtual void repaint (const SRect &rect) = 0;
+    virtual void video () = 0;
+    void remove ();                // remove from parent, mark ancestors dirty
+    void markDirty ();             // mark this and ancestors dirty
 
     NodePtrW node;
     SRect bounds;                  // bounds in in parent coord. 
-    Single xoffset, yoffset;       // translation for aspects
     float xscale, yscale;          // internal scaling
     unsigned int background_color; // rgba background color
+    bool dirty;                    // a decendant is removed
+    cairo_surface_t *surface;
 protected:
     Surface (const SRect & rect);
 };
@@ -805,6 +801,7 @@ namespace SMIL {
     class RegionBase;
     class Region;
     class Layout;
+    class Transition;
     class MediaType;
     class ImageMediaType;
     class TextMediaType;
@@ -833,6 +830,7 @@ public:
     virtual void visit (Node *) {}
     virtual void visit (SMIL::Region *);
     virtual void visit (SMIL::Layout *);
+    virtual void visit (SMIL::Transition *);
     virtual void visit (SMIL::TimedMrl *);
     virtual void visit (SMIL::MediaType *);
     virtual void visit (SMIL::ImageMediaType *);
