@@ -1051,7 +1051,7 @@ KMPLAYER_NO_EXPORT bool OssoMediaServer::brightness (int val, bool /*absolute*/)
 
 KMPLAYER_NO_EXPORT
 void OssoMediaServer::setVideoWindow (int x, int y, int w, int h) {
-    debugLog() << "OssoMediaServer::setVideoWindow " << w << "x" << h << endl;
+    //debugLog() << "OssoMediaServer::setVideoWindow " << w << "x" << h << endl;
     if (!(x < 0 || y < 0 || w <= 0 || h <= 1 /* hack !!*/ ||
                 x > 800 || y > 480 || w > 800 || h > 480))
         m_rect = Rect (x, y, w, h);
@@ -1388,7 +1388,7 @@ KMPLAYER_NO_EXPORT bool NpPlayer::quit () {
 
 KMPLAYER_NO_EXPORT
 void NpPlayer::setVideoWindow (int x, int y, int w, int h) {
-    debugLog() << "NpPlayer::setVideoWindow " << w << "x" << h << endl;
+    //debugLog() << "NpPlayer::setVideoWindow " << w << "x" << h << endl;
     if (!(x < 0 || y < 0 || w <= 0 || h <= 1 /* hack !!*/ ||
                 x > 800 || y > 480 || w > 800 || h > 480))
         nppInstanceWindowDimension (npdata, x, y, w, h);
