@@ -59,6 +59,10 @@
 
 struct GnomeVFSURI;
 
+namespace {
+    class GnomeVFSFilePrivate;
+}
+
 namespace KMPlayer {
 
 /**
@@ -78,6 +82,7 @@ class KMPLAYER_NO_EXPORT Single {
     friend float operator * (const float f, const Single s);
     friend double operator * (const double d, const Single s);
     friend Single operator / (const Single s, const int i);
+    friend float operator / (const Single s, const float f);
     friend double operator / (const Single s, const double d);
     friend double operator / (const double d, const Single s);
     friend bool operator > (const Single s1, const Single s2);
@@ -97,17 +102,18 @@ public:
     Single (const int v) : value (v << 8) {}
     Single (const float v) : value (int (256 * v)) {}
     Single (const double v) : value (int (256 * v)) {}
-    Single & operator = (const Single & s) { value = s.value; return *this; }
+    Single & operator = (const Single s) { value = s.value; return *this; }
     Single & operator = (const int v) { value = v << 8; return *this; }
     Single & operator = (const float v) { value = int (256 * v); return *this; }
     Single & operator = (const double v) { value = int(256 * v); return *this; }
-    Single & operator += (const Single & s) { value += s.value; return *this; }
+    Single & operator += (const Single s) { value += s.value; return *this; }
     Single & operator += (const int i) { value += (i << 8); return *this; }
-    Single & operator -= (const Single & s) { value -= s.value; return *this; }
+    Single & operator -= (const Single s) { value -= s.value; return *this; }
     Single & operator -= (const int i) { value -= (i << 8); return *this; }
-    Single & operator *= (const Single & s);
+    Single & operator *= (const Single s);
     Single & operator *= (const float f) { value = int(value*f); return *this; }
     Single & operator /= (const int i) { value /= i; return *this; }
+    Single & operator /= (const float f);
     operator int () const { return value >> 8; }
     operator double () const { return 1.0 * value / 256; }
 };
@@ -125,6 +131,7 @@ public:
     Matrix (Single xoff, Single yoff, float xscale, float yscale);
     void getXY (Single & x, Single & y) const;
     void getXYWH (Single & x, Single & y, Single & w, Single & h) const;
+    void invXYWH (Single & x, Single & y, Single & w, Single & h) const;
     void transform (const Matrix & matrix);
     void scale (float sx, float sy);
     void translate (Single x, Single y);
@@ -148,6 +155,21 @@ public:
     bool operator != (const SRect & r) const;
 };
 
+class KMPLAYER_NO_EXPORT IRect {
+public:
+    int x, y, w, h;
+    IRect ()
+        : x (0), y (0), w (0), h (0) {}
+    explicit IRect (const SRect r)
+        : x (r.x ()), y (r.y ()), w (r.width ()), h (r.height ()) {}
+    IRect (int a, int b, int c, int d)
+        : x (a), y (b), w (c), h (d) {}
+    IRect unite (const IRect & r) const;
+    IRect intersect (const IRect & r) const;
+    bool isValid () const { return w >= 0 && h >= 0; }
+    bool isEmpty () const { return w < 0 || h < 0; }
+};
+
 template <class T>
 class KMPLAYER_EXPORT CharTmpl {
     T m_unichar;
@@ -265,6 +287,7 @@ public:
     bool hasSuffix (const StringTmpl <T> & suffix) const;
     int find (const StringTmpl <T> & needle) const;
     int findRev (const StringTmpl <T> & needle) const;
+    StringTmpl <T> &replace (Char c1, Char c2);
     bool isEmpty () const;
     bool isNull () const;
     int length () const;
@@ -333,6 +356,7 @@ public:
     bool hasSuffix (const GLibString & suffix) const;
     int find (const GLibString & needle) const;
     int findRev (const GLibString & needle) const;
+    GLibString replace (Char c1, Char c2) const;
     bool isEmpty () const;
     GLibString mid (const int index, unsigned length=(unsigned)-1) const;
     GLibString left (int index) const;
@@ -362,8 +386,6 @@ public:
     off_t size ();
 };
 
-class GnomeVFSFilePrivate;
-
 class GnomeVFSFile {
     GnomeVFSFilePrivate * d;
 public:
@@ -556,6 +578,9 @@ public:
     Log & operator << (int i);
     Log & operator << (unsigned int i);
     Log & operator << (double d);
+    Log & operator << (const Single &s);
+    Log & operator << (const SRect &s);
+    Log & operator << (const IRect &s);
     Log & operator << (void *);
     Log & operator << (LOGFUNC manip);
 };
@@ -652,6 +677,7 @@ template <class T> class ImageTmpl;
 class GPixbuf;
 typedef ImageTmpl <GPixbuf> Image;
 
+#if 0
 template <class T>
 class PixmapTmpl {
     T m_pixmap;
@@ -691,7 +717,7 @@ public:
 };
 
 typedef PixmapTmpl <GPixmap> Pixmap;
-
+#endif
 
 template <class T>
 class ImageTmpl {
@@ -709,7 +735,7 @@ public:
     void reset ();
     ImageTmpl <T> scale (int w, int h);
     ImageTmpl <T> & operator = (const ImageTmpl <T> & other);
-    ImageTmpl <T> & operator = (const Pixmap & pixmap);
+    //ImageTmpl <T> & operator = (const Pixmap & pixmap);
     operator const T & () const;
 };
 
@@ -730,7 +756,7 @@ public:
     GPixbuf scale (int w, int h);
     operator GdkPixbuf * () const;
     GPixbuf & operator = (const GPixbuf & other);
-    GPixbuf & operator = (const Pixmap & pixmap);
+    //GPixbuf & operator = (const Pixmap & pixmap);
     GPixbuf & operator = (GdkPixbuf * pixbuf);
     GdkPixbuf * pixbuf () const;
 };
@@ -788,11 +814,16 @@ typedef PainterTmpl <GPainter> Painter;
 //-----------------------------------------------------------------------------
 
 
-inline Single & Single::operator *= (const Single & s) {
+inline Single & Single::operator *= (const Single s) {
     value = (((int64_t)value) * s.value) >> 8;
     return *this;
 }
 
+inline Single & Single::operator /= (const float f) {
+    value = (int) (value / f);
+    return *this;
+}
+
 inline Single operator + (const Single s1, const Single s2) {
     Single s;
     s.value = s1.value + s2.value;
@@ -856,6 +887,10 @@ inline Single operator / (const Single s, const int i) {
     return s1;
 }
 
+inline float operator / (const Single s, const float f) {
+    return (s.value / f ) / 256;
+}
+
 inline double operator / (const Single s, const double d) {
     return (s.value / d ) / 256;
 }
@@ -1291,6 +1326,12 @@ inline int StringTmpl<T>::findRev (const StringTmpl <T> & needle) const {
     return string.findRev (needle.string);
 }
 
+template <class T>
+inline StringTmpl <T> &StringTmpl<T>::replace (Char c1, Char c2) {
+    string = string.replace (c1, c2);
+    return *this;
+}
+
 template <class T>
 inline bool StringTmpl<T>::isEmpty () const {
     return string.isEmpty ();
@@ -1716,7 +1757,7 @@ template <class T> inline unsigned ColorTmpl <T>::rgb () const {
 }
 
 //-----------------------------------------------------------------------------
-
+#if 0
 template <class T> inline PixmapTmpl <T>::PixmapTmpl () {}
 
 template <class T>
@@ -1752,7 +1793,7 @@ template <class T> inline void PixmapTmpl <T>::fill (const Color & c) {
 template <class T> inline PixmapTmpl <T>::operator const T & () const {
     return m_pixmap;
 }
-
+#endif
 //-----------------------------------------------------------------------------
 
 template <class T> inline ImageTmpl <T>::ImageTmpl () {}
@@ -1795,13 +1836,13 @@ inline ImageTmpl <T> & ImageTmpl <T>::operator = (const ImageTmpl <T> & i) {
     m_image = i.m_image;
     return *this;
 }
-
+#if 0
 template <class T>
 inline ImageTmpl <T> & ImageTmpl <T>::operator = (const Pixmap & pm) {
     m_image = pm;
     return *this;
 }
-
+#endif
 template <class T> inline ImageTmpl <T>::operator const T & () const {
     return m_image;
 }
@@ -2162,7 +2203,7 @@ KMPLAYER_NO_EXPORT inline unsigned GColor::rgb () const {
 }
 
 //-----------------------------------------------------------------------------
-
+#if 0
 KMPLAYER_NO_EXPORT inline GPixmap::GPixmap () : m_pixmap (0L) {}
 
 KMPLAYER_NO_EXPORT inline bool GPixmap::isNull () const { return !m_pixmap; }
@@ -2172,7 +2213,7 @@ GPixmap::operator GdkDrawable * () const { return GDK_DRAWABLE (m_pixmap); }
 
 KMPLAYER_NO_EXPORT
 inline GdkPixmap * GPixmap::pixmap () const { return m_pixmap; }
-
+#endif
 //-----------------------------------------------------------------------------
 
 KMPLAYER_NO_EXPORT inline GPixbuf::GPixbuf () : m_pixbuf (0L) {}
